@@ -144,6 +144,42 @@
 		case LIBMTP_FILETYPE_FIRMWARE:
 			return @"firmware";
 			break;
+    case LIBMTP_FILETYPE_AAC:
+      return @"AAC";
+      break;
+    case LIBMTP_FILETYPE_MEDIACARD:
+      return @"Mediacard";
+      break;
+    case LIBMTP_FILETYPE_FLAC:
+      return @"FLAC";
+      break;
+    case LIBMTP_FILETYPE_MP2:
+      return @"MP2";
+      break;
+    case LIBMTP_FILETYPE_M4A:
+      return @"M4A";
+      break;
+    case LIBMTP_FILETYPE_DOC:
+      return @"DOC";
+      break;
+    case LIBMTP_FILETYPE_XML:
+      return @"XML";
+      break;
+    case LIBMTP_FILETYPE_XLS:
+      return @"XLS";
+      break;
+    case LIBMTP_FILETYPE_PPT:
+      return @"PPT";
+      break;
+    case LIBMTP_FILETYPE_MHT:
+      return @"MHT";
+      break;
+    case LIBMTP_FILETYPE_JP2:
+      return @"JP2";
+      break;
+    case LIBMTP_FILETYPE_JPX:
+      return @"JPX";
+      break;
 	}
 	// we can't ever get here...
 	NSLog(@"invalid libmtp file type %d", fileType);
@@ -199,6 +235,28 @@
 		[self setFileType:LIBMTP_FILETYPE_TEXT];
 	else if ([extension compare:@"HTM" options:NSCaseInsensitiveSearch] == NSOrderedSame || [extension compare:@"HTML" options:NSCaseInsensitiveSearch] == NSOrderedSame)
 		[self setFileType:LIBMTP_FILETYPE_HTML];
+  else if ([extension compare:@"AAC" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_AAC];
+  else if ([extension compare:@"FLAC" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_FLAC];
+  else if ([extension compare:@"MP2" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_MP2];
+  else if ([extension compare:@"M4A" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_M4A];
+  else if ([extension compare:@"DOC" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_DOC];
+  else if ([extension compare:@"XML" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_XML];
+  else if ([extension compare:@"XLS" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_XLS];
+  else if ([extension compare:@"PPT" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_PPT];
+  else if ([extension compare:@"MHT" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_MHT];
+  else if ([extension compare:@"JP2" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_JP2];
+  else if ([extension compare:@"JPX" options:NSCaseInsensitiveSearch] == NSOrderedSame)
+		[self setFileType:LIBMTP_FILETYPE_JPX];
 	else 
 		[self setFileType:LIBMTP_FILETYPE_UNKNOWN];
 }
