@@ -372,7 +372,7 @@ StatusDisplayer *statusDisplayerGlobal;
 			NSLog(@"Could not get battery level property description");
       NSString *errorString = [self njbErrorString];
       NSLog(errorString);
-      return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not get battry level property description" extendedErrorString:errorString] autorelease];
+      return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not get battery level property description" extendedErrorString:errorString] autorelease];
 		}
 		
 		LIBMTP_Get_Supported_Filetypes(device, &supportedFiletypes, &supportedFiletypesLength);
@@ -1984,12 +1984,12 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 	}
 }
 
-- (void)addTrack:(Track *)track ToAlbumWithName:(NSString *)albumName
+- (Album *)addTrack:(Track *)track ToAlbumWithName:(NSString *)albumName
 {
   NSLog (@"adding track with id %d to album name %@", [track itemID], albumName);
   // don't add to a blank album
   if ([albumName isEqualToString:@""])
-    return;
+    return nil;
   
   if (albums == nil)
     [self downloadAlbumList];
@@ -2017,6 +2017,12 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
   {
     // add track to existing album
     [album addTrack:track];
+    
+    if ([album albumID] == 0)
+    {
+      NSLog(@"Found album, but has 0 ID, giving up.");
+      return nil;
+    }
   }
   
   
@@ -2046,10 +2052,23 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
       NSString *errorString = [self njbErrorString];
       NSLog(errorString);
       
-      return;
+      [albums removeObject:album];
+      
+      // note it will think this has succeeded since we have to way of returning error yet.
+      // used to be a bug here: if we failed to make a new album, we'd keep the album in the list
+      // then try to add to it.  Now we remove it :).
+      
+      return nil;
     }
     else
     {
+      if (libmtpAlbum->album_id == 0)
+      {
+        NSLog(@"Created new album supposedly correctly, but got ID 0 back.. bailing out");
+        [albums removeObject:album];
+        return nil;
+      }
+        
       // let's set the artist too if we can
       
       uint32_t propnum = 0;
@@ -2066,7 +2085,7 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
         NSString *errorString = [self njbErrorString];
         NSLog(errorString);
         
-        return;
+        return nil;
       }
       
       for (propcounter = 0; propcounter < propnum; propcounter++)
@@ -2102,12 +2121,77 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
     {
       LIBMTP_destroy_album_t(libmtpAlbum);
       NSLog(@"Could not update album");
-      return;
+      return nil;
     }
   }
   
   [album setAlbumID:libmtpAlbum->album_id];
+
+  if (libmtpAlbum->album_id == 0)
+    NSLog(@"the album_id is 0!");
+
   LIBMTP_destroy_album_t(libmtpAlbum);
+  
+  return album;
+}
+
+- (NJBTransactionResult *)uploadAlbumArt:(NSString *)filename forTrack:(Track *)track
+{
+  NSString *albumName = [track album];
+  
+  // don't add to a blank album
+  if ([albumName isEqualToString:@""])
+    return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Cannot create album for blank album name"] autorelease];;
+  
+  if (albums == nil)
+    [self downloadAlbumList];
+  
+  Album *album;
+  NSEnumerator *albumEnumerator = [albums objectEnumerator];
+  
+  while (album = [albumEnumerator nextObject])
+  {
+    if ([[album name] caseInsensitiveCompare:albumName] == NSOrderedSame)
+      break;
+  }
+  
+  // TODO: if track is not in the album already, but the album exists, we need to add it
+  // we should also add an option that makes all the album files correctly
+  // need to warn if too big - can we rescale?  Can we read from tags?  iTunes?
+  // need to add @"Uploading Album Art" to localized strings
+  // update UI for all languages
+  // disable album art button for local files, or add album art to local tags (hard probably)
+  
+  if (album == nil)
+  {
+    album = [self addTrack:track ToAlbumWithName:albumName];
+    if (album == nil)
+      return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not add track to album"] autorelease];
+  }
+  
+  // TODO: worry about thread safety of file stuff
+  NSFileHandle *file = [NSFileHandle fileHandleForReadingAtPath:filename];
+  NSData *data = [file readDataToEndOfFile];
+  const char *bytes = [data bytes];
+  
+  LIBMTP_filesampledata_t sampledata;
+  sampledata.data = (char *)bytes;
+  sampledata.size = [data length];
+  
+  int ret = LIBMTP_Send_Representative_Sample(device, [album albumID], &sampledata);
+  
+  [file closeFile];
+  
+  if (ret != 0)
+  {
+    NSString *error = @"Could not send representative sample data";
+    NSLog(error);
+    NSString *errorString = [self njbErrorString];
+    NSLog(errorString);
+    return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error extendedErrorString:errorString] autorelease];
+  }
+  
+  return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
 @end
