@@ -18,17 +18,23 @@
 
 - (id)initWithSuccess:(BOOL)newSuccess
 {
-	return [self initWithSuccess:newSuccess resultString:@""];
+	return [self initWithSuccess:newSuccess resultString:@"" extendedErrorString:@""];
 }
 
 - (id)initWithSuccess:(BOOL)newSuccess resultString:(NSString *)newResultString
 {
-	if (self = [super init])
-	{
-		[self setSuccess:newSuccess];
+	return [self initWithSuccess:newSuccess resultString:newResultString extendedErrorString:@""];
+}
+
+- (id)initWithSuccess:(BOOL)newSuccess resultString:(NSString *)newResultString extendedErrorString:(NSString *)newExtendedErrorString
+{
+  if (self = [super init])
+  {
+    [self setSuccess:newSuccess];
 		[self setResultString:newResultString];
-	}
-	return self;
+    [self setExtendedErrorString:newExtendedErrorString];
+  }
+  return self;
 }
 
 - (void)dealloc
@@ -65,4 +71,20 @@
 	return resultString;
 }
 
+- (void)setExtendedErrorString:(NSString *)newExtendedErrorString
+{
+  if (newExtendedErrorString == nil)
+		newExtendedErrorString = @"";
+	// get immutable copy
+	newExtendedErrorString = [NSString stringWithString:newExtendedErrorString];
+	[newExtendedErrorString retain];
+	[extendedErrorString release];
+	extendedErrorString = newExtendedErrorString;
+}
+
+- (NSString *)extendedErrorString
+{
+  return extendedErrorString;
+}
+
 @end
