@@ -181,6 +181,8 @@
 			return [NSString stringWithString:NSLocalizedString(@"Deleting Playlist", nil)];
 		case STATUS_CREATING_FOLDER:
 			return [NSString stringWithString:NSLocalizedString(@"Creating Folder", nil)];
+    case STATUS_UPLOADING_ALBUM_ART:
+      return [NSString stringWithString:NSLocalizedString(@"Uploading Album Art", nil)];
 		case STATUS_UNKNOWN:
 		default:
 			NSLog(@"Unknown status %d in StatusDisplayer.statusStringForStatus", status);
