@@ -42,7 +42,7 @@
 #include "ptp-pack.c"
 
 /* To enable debug prints, switch on this */
-//#define ENABLE_USB_BULK_DEBUG
+#define ENABLE_USB_BULK_DEBUG
 
 /* OUR APPLICATION USB URB (2MB) ;) */
 #define PTPCAM_USB_URB		2097152
@@ -123,6 +123,9 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   { "Samsung YP-F2J", 0x04e8, 0x5057, DEVICE_FLAG_UNLOAD_DRIVER },
   // Reported by Patrick <skibler@gmail.com>
   { "Samsung YP-K5", 0x04e8, 0x505a, DEVICE_FLAG_NO_ZERO_READS },
+  // From dev.local@gmail.com - 0x4e8/0x507c is the UMS mode, don't add this.
+  // From m.eik michalke
+  { "Samsung YP-U3", 0x04e8, 0x507d, DEVICE_FLAG_NONE },
   // Reported by Matthew Wilcox <matthew@wil.cx>
   { "Samsung YP-T9", 0x04e8, 0x507f, DEVICE_FLAG_NONE },
   // From Paul Clinch
@@ -158,6 +161,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   { "Philips GoGear SA9200", 0x0471, 0x014f, DEVICE_FLAG_NONE },
   // From Gerhard Mekenkamp
   { "Philips GoGear Audio", 0x0471, 0x0165, DEVICE_FLAG_NONE },
+  // from David Holm <wormie@alberg.dk>
+  { "Philips Shoqbox", 0x0471, 0x0172, DEVICE_FLAG_ONLY_7BIT_FILENAMES },
   // from npedrosa
   { "Philips PSA610", 0x0471, 0x0181, DEVICE_FLAG_NONE },
   // From libgphoto2 source
@@ -170,26 +175,37 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
    * SanDisk
    * several devices (c150 for sure) are definately dual-mode and must 
    * have the USB mass storage driver that hooks them unloaded first.
+   * They all have problematic dual-mode making the device unload effect
+   * uncertain on these devices. All except for the Linux based ones seem
+   * to need DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL.
    */
   // Reported by Brian Robison
-  { "SanDisk Sansa m230/m240", 0x0781, 0x7400, DEVICE_FLAG_UNLOAD_DRIVER },
+  { "SanDisk Sansa m230/m240", 0x0781, 0x7400, 
+    DEVICE_FLAG_UNLOAD_DRIVER | DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL},
   // Reported by tangent_@users.sourceforge.net
-  { "SanDisk Sansa c150", 0x0781, 0x7410, DEVICE_FLAG_UNLOAD_DRIVER },
+  { "SanDisk Sansa c150", 0x0781, 0x7410, 
+    DEVICE_FLAG_UNLOAD_DRIVER | DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL },
   // From libgphoto2 source
   // Reported by <gonkflea@users.sourceforge.net>
   // Reported by Mike Owen <mikeowen@computerbaseusa.com>
-  { "SanDisk Sansa e200/e250/e260/e270/e280", 0x0781, 0x7420, DEVICE_FLAG_UNLOAD_DRIVER },
+  { "SanDisk Sansa e200/e250/e260/e270/e280", 0x0781, 0x7420, 
+    DEVICE_FLAG_UNLOAD_DRIVER |  DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL },
   // Reported by XNJB user
-  { "SanDisk Sansa e280", 0x0781, 0x7421, DEVICE_FLAG_UNLOAD_DRIVER },
+  { "SanDisk Sansa e280", 0x0781, 0x7421, 
+    DEVICE_FLAG_UNLOAD_DRIVER | DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL },
   // Reported by anonymous user at sourceforge.net
-  { "SanDisk Sansa c250", 0x0781, 0x7450, DEVICE_FLAG_UNLOAD_DRIVER },
+  { "SanDisk Sansa c240/c250", 0x0781, 0x7450, 
+    DEVICE_FLAG_UNLOAD_DRIVER |  DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL },
   // Reported by XNJB user, and Miguel de Icaza <miguel@gnome.org>
   // This has no dual-mode so no need to unload any driver.
   // This is a Linux based device!
   { "SanDisk Sansa Connect", 0x0781, 0x7480, DEVICE_FLAG_NONE },
   // Reported by Troy Curtis Jr.
-  { "SanDisk Sansa Express", 0x0781, 0x7460, DEVICE_FLAG_UNLOAD_DRIVER |
-                                             DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST },
+  { "SanDisk Sansa Express", 0x0781, 0x7460, 
+    DEVICE_FLAG_UNLOAD_DRIVER | DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST },
+  // Reported by XNJB user
+  { "SanDisk Sansa m240", 0x0781, 0x7430, 
+    DEVICE_FLAG_UNLOAD_DRIVER |  DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL },
   
 
   /*
@@ -252,6 +268,10 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   { "iRiver X20", 0x4102, 0x1132, 
     DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST | DEVICE_FLAG_NO_ZERO_READS | 
     DEVICE_FLAG_IRIVER_OGG_ALZHEIMER },
+  // Reported by Robert Ugo <robert_ugo@users.sourceforge.net>
+  { "iRiver T60", 0x4102, 0x1134, 
+    DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST | DEVICE_FLAG_NO_ZERO_READS | 
+    DEVICE_FLAG_IRIVER_OGG_ALZHEIMER },
   // Reported by Scott Call
   { "iRiver H10 20GB", 0x4102, 0x2101, 
     DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST | DEVICE_FLAG_NO_ZERO_READS | 
@@ -274,22 +294,28 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   // Reported by Nicholas Tripp
   { "Toshiba Gigabeat P20", 0x0930, 0x000f, DEVICE_FLAG_NONE },
   // From libgphoto2
-  { "Toshiba Gigabeat S", 0x0930, 0x0010, DEVICE_FLAG_NONE },
+  { "Toshiba Gigabeat S", 0x0930, 0x0010, DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL },
   // Reported by Rob Brown
   { "Toshiba Gigabeat P10", 0x0930, 0x0011, DEVICE_FLAG_NONE },
+  // Reported by Michael Davis <slithy@yahoo.com>
+  { "Toshiba Gigabeat U", 0x0930, 0x0016, DEVICE_FLAG_NONE },
   
   /*
    * Archos
+   * These devices have some dual-mode interfaces which will really
+   * respect the driver unloading, so DEVICE_FLAG_UNLOAD_DRIVER
+   * really work on these devices!
    */
-  // Reported by gudul1@users.sourceforge.net
-  { "Archos 104 (MTP mode)", 0x0e79, 0x120a, DEVICE_FLAG_NONE },
-  // Added by Jan Binder
-  { "Archos XS202 (MTP mode)", 0x0e79, 0x1208, DEVICE_FLAG_NONE },
-  // Reported by Etienne Chauchot <chauchot.etienne@free.fr>
-  // This seems to be dual-mode and thus need to have the unload flag.
-  { "Archos 504 (MTP mode)", 0x0e79, 0x1307, DEVICE_FLAG_UNLOAD_DRIVER },
   // Reported by Alexander Haertig <AlexanderHaertig@gmx.de>
   { "Archos Gmini XS100", 0x0e79, 0x1207, DEVICE_FLAG_UNLOAD_DRIVER },
+  // Added by Jan Binder
+  { "Archos XS202 (MTP mode)", 0x0e79, 0x1208, DEVICE_FLAG_NONE },
+  // Reported by gudul1@users.sourceforge.net
+  { "Archos 104 (MTP mode)", 0x0e79, 0x120a, DEVICE_FLAG_NONE },
+  // Reported by Etienne Chauchot <chauchot.etienne@free.fr>
+  { "Archos 504 (MTP mode)", 0x0e79, 0x1307, DEVICE_FLAG_UNLOAD_DRIVER },
+  // Reported by Kay McCormick <kaym@modsystems.com>
+  { "Archos 704 mobile dvr", 0x0e79, 0x130d, DEVICE_FLAG_UNLOAD_DRIVER },
 
   /*
    * Dunlop (OEM of EGOMAN ltd?) reported by Nanomad
@@ -324,7 +350,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
    */
   // From: Mitchell Hicks <mitchix@yahoo.com>
   {"Nokia 5300 Mobile Phone", 0x0421, 0x04ba, DEVICE_FLAG_NONE },
-  {"Some Nokia Mobile Phone (MTP mode)", 0x0421, 0x04e1, DEVICE_FLAG_NONE },
+  // From Swapan <swapan@yahoo.com>
+  {"Nokia N75 Mobile Phone", 0x0421, 0x04e1, DEVICE_FLAG_NONE },
   // From: Pat Nicholls <pat@patandannie.co.uk>
   {"Nokia N80 Internet Edition (Media Player)", 0x0421, 0x04f1, DEVICE_FLAG_UNLOAD_DRIVER },
 
@@ -355,6 +382,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
    * Their datasheet claims their devices are dualmode so probably needs to
    * unload the attached drivers here.
    */
+  // Reported by Cristi Magherusan <majeru@gentoo.ro>
+  {"TrekStor Vibez i.Beat sweez FM", 0x0402, 0x0611, DEVICE_FLAG_UNLOAD_DRIVER },
   // Reported by Stefan Voss <svoss@web.de>
   {"TrekStor Vibez 8/12GB", 0x066f, 0x842a, DEVICE_FLAG_UNLOAD_DRIVER },
   
@@ -364,6 +393,22 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   // Reported by XNJB user
   {"Disney MixMax", 0x0aa6, 0x6021, DEVICE_FLAG_NONE },
 
+  /*
+   * Cowon Systems, Inc.
+   * The iAudio audiophile devices don't encourage the use of MTP.
+   */
+  // Reported by Patrik Johansson <Patrik.Johansson@qivalue.com>
+  {"Cowon iAudio U3 (MTP mode)", 0x0e21, 0x0701, DEVICE_FLAG_NONE },
+  // Reported by Roberth Karman
+  {"Cowon iAudio 7 (MTP mode)", 0x0e21, 0x0751, DEVICE_FLAG_NONE },
+  // Reported by TJ Something <tjbk_tjb@users.sourceforge.net>
+  {"Cowon iAudio D2 (MTP mode)", 0x0e21, 0x0801, DEVICE_FLAG_NONE },
+
+  /*
+   * Insignia, dual-mode.
+   */
+  { "Insignia NS-DV45", 0x19ff, 0x0303, DEVICE_FLAG_UNLOAD_DRIVER },
+
   /*
    * Other strange stuff.
    */
