@@ -32,6 +32,15 @@ htod32ap (PTPParams *params, unsigned char *a, uint32_t val)
 		htobe32a(a,val);
 }
 
+static inline void
+htod64ap (PTPParams *params, unsigned char *a, uint64_t val)
+{
+	if (params->byteorder==PTP_DL_LE)
+		htole64a(a,val);
+	else 
+		htobe64a(a,val);
+}
+
 static inline uint16_t
 dtoh16p (PTPParams *params, uint16_t var)
 {
@@ -44,6 +53,12 @@ dtoh32p (PTPParams *params, uint32_t var)
 	return ((params->byteorder==PTP_DL_LE)?le32toh(var):be32toh(var));
 }
 
+static inline uint64_t
+dtoh64p (PTPParams *params, uint64_t var)
+{
+	return ((params->byteorder==PTP_DL_LE)?le64toh(var):be64toh(var));
+}
+
 static inline uint16_t
 dtoh16ap (PTPParams *params, unsigned char *a)
 {
@@ -59,24 +74,16 @@ dtoh32ap (PTPParams *params, unsigned char *a)
 static inline uint64_t
 dtoh64ap (PTPParams *params, unsigned char *a)
 {
-	uint64_t tmp = 0;
-	int i;
-
-	if (params->byteorder==PTP_DL_LE) {
-		for (i=0;i<8;i++)
-			tmp |= (((uint64_t)a[i]) << (8*i));
-	} else {
-		for (i=0;i<8;i++)
-			tmp |= (((uint64_t)a[i]) << (8*(7-i)));
-	}
-	return tmp;
+	return ((params->byteorder==PTP_DL_LE)?le64atoh(a):be64atoh(a));
 }
 
 #define htod8a(a,x)	*(uint8_t*)(a) = x
 #define htod16a(a,x)	htod16ap(params,a,x)
 #define htod32a(a,x)	htod32ap(params,a,x)
+#define htod64a(a,x)	htod64ap(params,a,x)
 #define htod16(x)	htod16p(params,x)
 #define htod32(x)	htod32p(params,x)
+#define htod64(x)	htod64p(params,x)
 
 #define dtoh8a(x)	(*(uint8_t*)(x))
 #define dtoh16a(a)	dtoh16ap(params,a)
@@ -84,6 +91,7 @@ dtoh64ap (PTPParams *params, unsigned char *a)
 #define dtoh64a(a)	dtoh64ap(params,a)
 #define dtoh16(x)	dtoh16p(params,x)
 #define dtoh32(x)	dtoh32p(params,x)
+#define dtoh64(x)	dtoh64p(params,x)
 
 
 static inline char*
@@ -182,8 +190,12 @@ ptp_get_packed_stringcopy(PTPParams *params, char *string, uint32_t *packed_size
 	uint8_t packed[PTP_MAXSTRLEN*2+3], len;
 	size_t plen;
 	unsigned char *retcopy = NULL;
-
-	ptp_pack_string(params, string, (unsigned char*) packed, 0, &len);
+  
+	if (string == NULL)
+	  ptp_pack_string(params, "", (unsigned char*) packed, 0, &len);
+	else
+	  ptp_pack_string(params, string, (unsigned char*) packed, 0, &len);
+  
 	/* returned length is in characters, then one byte for string length */
 	plen = len*2 + 1;
 	
@@ -587,17 +599,13 @@ ptp_unpack_DPV (
 	case PTP_DTC_UINT32:
 		CTVAL(value->u32,dtoh32a);
 		break;
-
-
-
-	case PTP_DTC_UINT64:
-		*offset += 8;
-		/*fprintf(stderr,"unhandled unpack of uint64\n");*/
-		break;
 	case PTP_DTC_INT64:
-		*offset += 8;
-		/*fprintf(stderr,"unhandled unpack of int64\n");*/
+		CTVAL(value->i64,dtoh64a);
 		break;
+	case PTP_DTC_UINT64:
+		CTVAL(value->u64,dtoh64a);
+		break;
+
 	case PTP_DTC_UINT128:
 		*offset += 16;
 		/*fprintf(stderr,"unhandled unpack of uint128n");*/
@@ -627,6 +635,12 @@ ptp_unpack_DPV (
 	case PTP_DTC_AINT32:
 		RARR(value,i32,dtoh32a);
 		break;
+	case PTP_DTC_AUINT64:
+		RARR(value,u64,dtoh64a);
+		break;
+	case PTP_DTC_AINT64:
+		RARR(value,i64,dtoh64a);
+		break;
 	/* XXX: other int types are unimplemented */
 	/* XXX: other int arrays are unimplemented also */
 	case PTP_DTC_STR: {
@@ -830,47 +844,71 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr
 		dpv=malloc(size);
 		htod32a(dpv,value->u32);
 		break;
+	case PTP_DTC_INT64:
+		size=sizeof(int64_t);
+		dpv=malloc(size);
+		htod64a(dpv,value->i64);
+		break;
+	case PTP_DTC_UINT64:
+		size=sizeof(uint64_t);
+		dpv=malloc(size);
+		htod64a(dpv,value->u64);
+		break;
 	case PTP_DTC_AUINT8:
 		size=sizeof(uint32_t)+value->a.count*sizeof(uint8_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod8a(&dpv[4+i],value->a.v[i].u8);
+			htod8a(&dpv[sizeof(uint32_t)+i*sizeof(uint8_t)],value->a.v[i].u8);
 		break;
 	case PTP_DTC_AINT8:
 		size=sizeof(uint32_t)+value->a.count*sizeof(int8_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod8a(&dpv[4+i],value->a.v[i].i8);
+			htod8a(&dpv[sizeof(uint32_t)+i*sizeof(int8_t)],value->a.v[i].i8);
 		break;
 	case PTP_DTC_AUINT16:
 		size=sizeof(uint32_t)+value->a.count*sizeof(uint16_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod16a(&dpv[4+i],value->a.v[i].u16);
+			htod16a(&dpv[sizeof(uint32_t)+i*sizeof(uint16_t)],value->a.v[i].u16);
 		break;
 	case PTP_DTC_AINT16:
 		size=sizeof(uint32_t)+value->a.count*sizeof(int16_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod16a(&dpv[4+i],value->a.v[i].i16);
+			htod16a(&dpv[sizeof(uint32_t)+i*sizeof(int16_t)],value->a.v[i].i16);
 		break;
 	case PTP_DTC_AUINT32:
 		size=sizeof(uint32_t)+value->a.count*sizeof(uint32_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod32a(&dpv[4+i],value->a.v[i].u32);
+			htod32a(&dpv[sizeof(uint32_t)+i*sizeof(uint32_t)],value->a.v[i].u32);
 		break;
 	case PTP_DTC_AINT32:
 		size=sizeof(uint32_t)+value->a.count*sizeof(int32_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod32a(&dpv[4+i],value->a.v[i].i32);
+			htod32a(&dpv[sizeof(uint32_t)+i*sizeof(int32_t)],value->a.v[i].i32);
+		break;
+	case PTP_DTC_AUINT64:
+		size=sizeof(uint32_t)+value->a.count*sizeof(uint64_t);
+		dpv=malloc(size);
+		htod32a(dpv,value->a.count);
+		for (i=0;i<value->a.count;i++)
+			htod64a(&dpv[sizeof(uint32_t)+i*sizeof(uint64_t)],value->a.v[i].u64);
+		break;
+	case PTP_DTC_AINT64:
+		size=sizeof(uint32_t)+value->a.count*sizeof(int64_t);
+		dpv=malloc(size);
+		htod32a(dpv,value->a.count);
+		for (i=0;i<value->a.count;i++)
+			htod64a(&dpv[sizeof(uint32_t)+i*sizeof(int64_t)],value->a.v[i].i64);
 		break;
 	/* XXX: other int types are unimplemented */
 	case PTP_DTC_STR: {
@@ -1104,17 +1142,17 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 			int		j;
 			PTPDevicePropDesc	*dpd;
 
-			fprintf (stderr, "Adding EOS property %04x desc record, datasize is %d\n", proptype, size-PTP_ece_Prop_Desc_Data);
+			/*fprintf (stderr, "Adding EOS property %04x desc record, datasize is %d\n", proptype, size-PTP_ece_Prop_Desc_Data);*/
 			for (j=0;j<params->nrofcanon_props;j++)
 				if (params->canon_props[j].proptype == proptype)
 					break;
 			if (j==params->nrofcanon_props) {
-				fprintf (stderr, "should have received default value for %x first!\n", proptype);
+				/*fprintf (stderr, "should have received default value for %x first!\n", proptype);*/
 				break;
 			}
 			dpd = &params->canon_props[j].dpd;
 			if (propxtype != 3) {
-				fprintf (stderr, "propxtype is %x for %x, unhandled.\n", propxtype, proptype);
+				/*fprintf (stderr, "propxtype is %x for %x, unhandled.\n", propxtype, proptype);*/
 				break;
 			}
 			if (propxcnt) {
@@ -1125,14 +1163,14 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 					switch (dpd->DataType) {
 					case PTP_DTC_UINT16:
 						dpd->FORM.Enum.SupportedValue[j].u16	= dtoh16a(data);
-						fprintf (stderr,"suppvalue[%d] of %x is %x\n", j, proptype, dtoh16a(data));
+						/*fprintf (stderr,"suppvalue[%d] of %x is %x\n", j, proptype, dtoh16a(data));*/
 						break;
 					case PTP_DTC_UINT8:
 						dpd->FORM.Enum.SupportedValue[j].u8	= dtoh8a(data);
-						fprintf (stderr,"suppvalue[%d] of %x is %x\n", j, proptype, dtoh8a(data));
+						/*fprintf (stderr,"suppvalue[%d] of %x is %x\n", j, proptype, dtoh8a(data));*/
 						break;
 					default:
-						fprintf(stderr,"data type 0x%04x of %x unhandled, fill in (val=%x).\n", dpd->DataType, proptype, dtoh32a(data));
+						/*fprintf(stderr,"data type 0x%04x of %x unhandled, fill in (val=%x).\n", dpd->DataType, proptype, dtoh32a(data));*/
 						break;
 					}
 					data += 4; /* might only be for propxtype 3 */
@@ -1173,6 +1211,9 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 				}
 				dpd = &params->canon_props[j].dpd;
 				switch (proptype) {
+				case PTP_DPC_CANON_EOS_CameraTime:
+					dpd->DataType = PTP_DTC_UINT32;
+					break;
 				case PTP_DPC_CANON_EOS_Aperture:
 				case PTP_DPC_CANON_EOS_ShutterSpeed:
 				case PTP_DPC_CANON_EOS_ISOSpeed:
@@ -1188,10 +1229,15 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 					dpd->DataType = PTP_DTC_STR;
 					break;
 				default:
-					fprintf (stderr, "Unknown EOS property %04x, datasize is %d\n", proptype, size-PTP_ece_Prop_Val_Data);
+					/*fprintf (stderr, "Unknown EOS property %04x, datasize is %d\n", proptype, size-PTP_ece_Prop_Val_Data);*/
 					break;
 				}
 				switch (dpd->DataType) {
+				case PTP_DTC_UINT32:
+					dpd->FactoryDefaultValue.u32	= dtoh32a(data);
+					dpd->CurrentValue.u32		= dtoh32a(data);
+					/*fprintf (stderr,"currentvalue of %x is %x\n", proptype, dpd->CurrentValue.u16);*/
+					break;
 				case PTP_DTC_UINT16:
 					dpd->FactoryDefaultValue.u16	= dtoh16a(data);
 					dpd->CurrentValue.u16		= dtoh16a(data);
@@ -1215,7 +1261,7 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 				break;
 		}
 		default:
-			fprintf (stderr, "unknown EOS property type %04x\n", type);
+			/* fprintf (stderr, "unknown EOS property type %04x\n", type); */
 			(*ce)[i].type = PTP_CANON_EOS_CHANGES_TYPE_UNKNOWN;
 			break;
 		}
